/**
 * 会费管理逻辑
 */

import moment = require("moment");
import { MEMBERLEVEL, MEMBERSTATE, MEMBERTYPE, ORDERSTATE, PAYMENTSTATUS, PAYMENTTYPE, STATE } from "../../config/enum";
import { TABLEENUM } from "../../data/models/model";
import { find, findCount, findOnce, findOnceToSort, findToPage, findToSort } from "../../data/select";
import { extractData } from "../../util/piecemeal";
import { changeEnumValue } from "../../util/verificationEnum";
import { BizError } from "../../util/bizError";
import { ERRORENUM } from "../../config/errorEnum";
import { updateOneData } from "../../data/update";
import { successResult } from "../../tools/system";



/**
 * 已缴费列表 success
 */
export async function getPaid({unitName, joinStartTime, joinEndTime, memberType, pageNumber}) {
    const NowMs = new Date().valueOf();
    /**到期时间距离今天大于90天 */
    let selectParam:any = {
        "$and":[
            { "lifespanEndTime":{"$gt":NowMs + (90 * 24 * 3600 * 1000)}},
            { "lifespanEndTime":{"$gt":NowMs}},
        ],
        isAdmin:STATE.否,
        isGracePeriod:STATE.否
    }

    if (unitName) selectParam.unitName = unitName;
    if (joinStartTime) {
        selectParam["$and"].push( { "joinTime":{"$gt":joinStartTime} } )
    }
    if (joinEndTime) {
        selectParam["$and"].push( { "joinTime":{"$lt":joinEndTime} } )
    }
    if (memberType) selectParam.memberType = memberType;

    const SelectFiles = ["userId", "memberState", "unitName", "loginId", "phone", "joinTime", "memberType", "lifespanStartTime", "lifespanEndTime", "memberLevel"];
    let dbList = await findToPage(TABLEENUM.用户表, selectParam, SelectFiles, pageNumber);
    let dataCount = await findCount(TABLEENUM.用户表, selectParam);

    let dataList = [];
    const itemFiles = ["userId", "memberState", "unitName", "loginId", "phone", "joinTime", "memberType"];
    dbList.forEach(info => {
        let item:any = extractData(info, itemFiles);
        item.lifespanTime = `${moment(info.lifespanStartTime).format("YYYY-MM-DD")} 至 ${moment(info.lifespanEndTime).format("YYYY-MM-DD")}`;
        item.nextLifespanTime = "大于90天";
        item.joinTime = moment(info.joinTime).format("YYYY-MM-DD");
        item.memberState = changeEnumValue(MEMBERSTATE, item.memberState);
        item.memberType = changeEnumValue(MEMBERTYPE, item.memberType);
        dataList.push(item);
    });

    return {dataList, dataCount};
}

/**
 * 已缴费列表中的订单日志 success
 * @param id 会员列表返回的userId  
 */
export async function paidOrderList({id}) {
    let dbList = await findToSort(TABLEENUM.订单表, {userId:id}, {payTime:-1}, ["orderCycleStart", "orderCycleEnd", "money", "paymentMethod", "state"]);

    let dataList = [];
    dbList.forEach(info => {
        let {orderCycleStart, orderCycleEnd, money, paymentMethod, state} = info;
        let paymentMethodStr = '-';
        if ( state == ORDERSTATE.已支付 ) {
            paymentMethodStr = changeEnumValue(PAYMENTTYPE, paymentMethod);
        }
        dataList.push({
            orderCycle:`${moment(orderCycleStart).format("YYYY-MM-DD")}至${moment(orderCycleEnd).format("YYYY-MM-DD")}`,
            money,
            paymentMethod:paymentMethodStr,
            state:changeEnumValue(ORDERSTATE, state)
        });
    });

    return {dataList};
}


/**
 * 待缴费列表 success
 * @param param0 
 */
export async function unpaidList({unitName, joinStartTime, joinEndTime, memberType, pageNumber}) {
    const NowMs = new Date().valueOf();
    /**到期时间距离今天大于90天 */
    let selectParam:any = {
        "$and":[
            { "lifespanEndTime":{"$lt":NowMs + (30 * 24 * 3600 * 1000)}},
            { "lifespanEndTime":{"$gt":NowMs}}
        ],
        isAdmin:STATE.否,
        isGracePeriod:STATE.否
    }

    if (unitName) selectParam.unitName = unitName;
    if (joinStartTime) {
        selectParam["$and"].push( { "joinTime":{"$gt":joinStartTime} } )
    }
    if (joinEndTime) {
        selectParam["$and"].push( { "joinTime":{"$lt":joinEndTime} } )
    }
    if (memberType) selectParam.memberType = memberType;

    const SelectFiles = ["userId", "memberState", "unitName", "loginId", "phone", "joinTime", "memberType", "lifespanStartTime", "lifespanEndTime"];
    let dbList = await findToPage(TABLEENUM.用户表, selectParam, SelectFiles, pageNumber, 10);
    let dataCount = await findCount(TABLEENUM.用户表, selectParam);

    let dataList = [];
    const itemFiles = ["userId", "memberState", "unitName", "loginId", "phone", "joinTime", "memberType"];
    dbList.forEach(info => {
        let item:any = extractData(info, itemFiles);
        item.lifespanTime = `${moment(info.lifespanStartTime).format("YYYY-MM-DD")} 至 ${moment(info.lifespanEndTime).format("YYYY-MM-DD")}`;
        item.nextLifespanTime = "不足90天";
        if (info.lifespanEndTime < ( NowMs + (30 * 24 * 3600 * 1000) ) ) item.nextLifespanTime = "不足30天";
    
        item.memberState = changeEnumValue(MEMBERSTATE, item.memberState);
        item.memberType = changeEnumValue(MEMBERTYPE, item.memberType);
        dataList.push(item);
    });

    return {dataList, dataCount};
}


/**
 * 会费管理-欠费 success
 */
export async function getRenewalPeriod({unitName, joinStartTime, joinEndTime, memberType, pageNumber}) {
    const NowMs = new Date().valueOf();
    /**到期时间距离今天大于90天 */
    let selectParam:any = {
        "$or":[
            { 
                isGracePeriod:STATE.否, 
                lifespanEndTime:{"$lt":NowMs}
            }, 
            {
                isGracePeriod:STATE.是, 
                lifespanEndTime:{"$lt":NowMs},
                gracePeriodEndTime:{"$lt":NowMs}
            }],
        isAdmin:STATE.否,
    }

    if (unitName) selectParam.unitName = unitName;
    if (joinStartTime) {
        if (!selectParam["$and"]) selectParam["$and"] = [];
        selectParam["$and"].push( { "joinTime":{"$gt":joinStartTime} } )
    }
    if (joinEndTime) {
        if (!selectParam["$and"]) selectParam["$and"] = [];
        selectParam["$and"].push( { "joinTime":{"$lt":joinEndTime} } )
    }
    if (memberType) selectParam.memberType = memberType;

    const SelectFiles = ["userId", "memberState", "unitName", "loginId", "phone", "joinTime", "memberType", "lifespanStartTime", "lifespanEndTime"];
    let dbList = await findToPage(TABLEENUM.用户表, selectParam, SelectFiles, pageNumber, 10);
    let dataCount = await findCount(TABLEENUM.用户表, selectParam);

    let dataList = [];
    const itemFiles = ["userId", "memberState", "unitName", "loginId", "phone", "joinTime", "memberType"];
    dbList.forEach(info => {
        let item:any = extractData(info, itemFiles);
        item.lifespanTime = `${moment(info.lifespanEndTime).format("YYYY-MM-DD")}`;
        if (info.lifespanEndTime > ( NowMs - (30 * 24 * 3600 * 1000) ) ) item.nextLifespanTime = "过期30天内";
        else if (info.lifespanEndTime > ( NowMs - (365 * 24 * 3600 * 1000) ) ) {
            item.nextLifespanTime = "过期1年内";
        } else  item.nextLifespanTime = "过期2年内";
    
        item.memberState = changeEnumValue(MEMBERSTATE, item.memberState);
        item.memberType = changeEnumValue(MEMBERTYPE, item.memberType);
        dataList.push(item);
    });

    return {dataList, dataCount};
}


/**
 * 线下付款 success
 * @param param0 
 */
export async function offlinePayment({orderId, paymentNum, offlinePaymentUrl }) {
    let orderInfo = await findOnce(TABLEENUM.订单表, {id:orderId});
    if (!orderInfo || !orderInfo.id) throw new BizError(ERRORENUM.目标数据不存在);
    if (orderInfo.state == ORDERSTATE.已支付) throw new BizError(ERRORENUM.该订单已支付);
    let notPayCount = await findCount(TABLEENUM.订单表, {userId:orderInfo.userId, state:ORDERSTATE.未支付} );

    let updateInfo = {
        paymentNum,
        offlinePaymentUrl,
        state:ORDERSTATE.已支付,
        paymentMethod:PAYMENTTYPE.线下支付,
        payTime:new Date().valueOf()
    };

    await updateOneData(TABLEENUM.订单表, {id:orderId}, updateInfo);
   
    /**只有一笔欠费时才更新用户状态 */
    if (notPayCount <= 1) {
        let newOrderInfo = await findOnceToSort(TABLEENUM.订单表, {userId:orderInfo.userId}, {lifespanEndTime:-1}, ["lifespanStartTime","lifespanEndTime"]);
        let updateUserInfo = {
            lifespanStartTime:newOrderInfo.lifespanStartTime,
            lifespanEndTime:newOrderInfo.lifespanEndTime,
            isGracePeriod:STATE.否,
            gracePeriodEndTime:0,
            memberState:MEMBERSTATE.正常,
            paymentStatus:PAYMENTSTATUS.已支付
        };
        await updateOneData(TABLEENUM.用户表, {userId:orderInfo.userId}, updateUserInfo);
    }

    return successResult();
}


/**
 * 多比订单时选择线下付款页面的选择缴费周期页
 * @param param0 
 * @returns 
 */
export async function selectManyOrderForUser({id}) {
    let notPayList = await find(TABLEENUM.订单表, {userId:id, state:ORDERSTATE.未支付},["orderCycleStart", "orderCycleEnd", "id"] );

    let dataList = [];

    notPayList.forEach(info => {
        let {orderCycleStart, orderCycleEnd, id} = info;

        dataList.push({
            key: `${moment(orderCycleStart).format("YYYY-MM-DD")} 至 ${moment(orderCycleEnd).format("YYYY-MM-DD")}`,
            value:id
        });
    });
    return {dataList};
}



/**
 * 设置宽限期 success
 * @param param0 
 */
export async function setGracePeriod({id, gracePeriodEndTime}) {
    let userInfo = await findOnce(TABLEENUM.用户表, {userId:id, isAdmin:STATE.否}, ["userId"]);
    if (!userInfo || !userInfo.userId) throw new BizError(ERRORENUM.目标数据不存在);

    let updateInfo = {
        isGracePeriod:STATE.是,
        gracePeriodEndTime
    };

    await updateOneData(TABLEENUM.用户表, {userId:id}, updateInfo);

    return successResult();
}


/**
 * 宽限期列表 success
 */
export async function getGracePeriod({unitName, joinStartTime, joinEndTime, memberType, pageNumber}) {
    const NowMs = new Date().valueOf();
    /**到期时间距离今天大于90天 */
    let selectParam:any = {
        isGracePeriod:STATE.是, 
        gracePeriodEndTime:{"$gt":NowMs},
        isAdmin:STATE.否,
    }

    if (unitName) selectParam.unitName = unitName;
    if (joinStartTime) {
        selectParam["$and"].push( { "joinTime":{"$gt":joinStartTime} } )
    }
    if (joinEndTime) {
        selectParam["$and"].push( { "joinTime":{"$lt":joinEndTime} } )
    }
    if (memberType) selectParam.memberType = memberType;

    const SelectFiles = ["userId", "memberState", "unitName", "loginId", "phone", "joinTime", "memberType", "lifespanStartTime", "lifespanEndTime"];
    let dbList = await findToPage(TABLEENUM.用户表, selectParam, SelectFiles, pageNumber, 10);
    let dataCount = await findCount(TABLEENUM.用户表, selectParam);

    let dataList = [];
    const itemFiles = ["userId", "memberState", "loginId", "phone", "joinTime", "memberType", "unitName"];
    dbList.forEach(info => {
        let dataInfo = {
            userId:info.userId,
            memberState:changeEnumValue(MEMBERSTATE, info.memberState),
            loginId:info.loginId,
            phone:info.phone,
            unitName,
            lifespanTime:`${moment(info.lifespanEndTime).format("YYYY-MM-DD")}`,
            nextLifespanTime:"宽限期中",
            memberType:changeEnumValue(MEMBERTYPE, info.memberType)
        };

        dataList.push(dataInfo);
    });

    return {dataList, dataCount};
}


/**
 * 移除宽限期
 * @param id 宽限期列表返回的userId  
 * @returns 
 */
export async function delGracePeriod({id}) {
    let userInfo = await findOnce(TABLEENUM.用户表, {userId:id, isAdmin:STATE.否}, ["userId"]);
    if (!userInfo || !userInfo.userId) throw new BizError(ERRORENUM.目标数据不存在);
    if (userInfo.isGracePeriod) throw new BizError(ERRORENUM.该用户已在宽限期内);

    let updateInfo = {
        isGracePeriod:STATE.否,
        gracePeriodEndTime:0
    };

    await updateOneData(TABLEENUM.用户表, {userId:id}, updateInfo);

    return successResult();
}